@@ -53,7 +53,7 @@ pub struct Writer<'a, W, X> {
     core_data_writer: &'a mut BitWriter<W>,
     external_data_writers: &'a mut HashMap<Itf8, X>,
     reference_sequence_id: ReferenceSequenceId,
-    prev_alignment_start: Itf8,
+    prev_alignment_start: Option<sam::record::Position>,
 }
 
 impl<'a, W, X> Writer<'a, W, X>
@@ -66,7 +66,7 @@ where
         core_data_writer: &'a mut BitWriter<W>,
         external_data_writers: &'a mut HashMap<Itf8, X>,
         reference_sequence_id: ReferenceSequenceId,
-        initial_alignment_start: Itf8,
+        initial_alignment_start: Option<sam::record::Position>,
     ) -> Self {
         Self {
             compression_header,
@@ -152,9 +152,24 @@ where
             .ap_data_series_delta();
 
         let alignment_start = if ap_data_series_delta {
-            record.alignment_start() - self.prev_alignment_start
+            match (record.alignment_start(), self.prev_alignment_start) {
+                (None, None) => 0,
+                (Some(alignment_start), Some(prev_alignment_start)) => {
+                    i32::from(alignment_start) - i32::from(prev_alignment_start)
+                }
+                _ => {
+                    return Err(io::Error::new(
+                        io::ErrorKind::InvalidData,
+                        format!(
+                            "invalid alignment start ({:?}) or previous alignment start ({:?})",
+                            record.alignment_start(),
+                            self.prev_alignment_start
+                        ),
+                    ));
+                }
+            }
         } else {
-            record.alignment_start()
+            record.alignment_start().map(i32::from).unwrap_or_default()
         };
 
         self.write_alignment_start(alignment_start)?;
@@ -326,9 +341,10 @@ where
 
     fn write_next_mate_alignment_start(
         &mut self,
-        next_mate_alignment_start: Itf8,
+        next_mate_alignment_start: Option<sam::record::Position>,
     ) -> io::Result<()> {
-        self.compression_header
+        let encoding = self
+            .compression_header
             .data_series_encoding_map()
             .next_mate_alignment_start_encoding()
             .ok_or_else(|| {
@@ -336,15 +352,16 @@ where
                     io::ErrorKind::InvalidData,
                     WriteRecordError::MissingDataSeriesEncoding(DataSeries::NextMateAlignmentStart),
                 )
-            })
-            .and_then(|encoding| {
-                encode_itf8(
-                    encoding,
-                    &mut self.core_data_writer,
-                    &mut self.external_data_writers,
-                    next_mate_alignment_start,
-                )
-            })
+            })?;
+
+        let position = next_mate_alignment_start.map(i32::from).unwrap_or_default();
+
+        encode_itf8(
+            encoding,
+            &mut self.core_data_writer,
+            &mut self.external_data_writers,
+            position,
+        )
     }
 
     fn write_template_size(&mut self, template_size: Itf8) -> io::Result<()> {
